<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2003-2010 Renzo Lauper (renzo@churchtool.org)
*  All rights reserved
*
*  This script is part of the kOOL project. The kOOL project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*  A copy is found in the textfile GPL.txt and important notices to the license
*  from the author is found in LICENSE.txt distributed with these scripts.
*
*  kOOL is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once($BASE_PATH."inc/class.kOOL_listview.php");


function ko_list_donations($output=TRUE, $mode="html", $dontApplyLimit=FALSE) {
	global $smarty;
	global $group_view, $group_new, $group_edit, $group_mod, $item_view, $item_edit;

	if(!$group_view && !$item_view) return;
	apply_donations_filter($z_where, $z_limit);
	if($dontApplyLimit) $z_limit = "";

	$order = "ORDER BY ".$_SESSION["sort_donations"]." ".$_SESSION["sort_donations_order"];

	$rows = db_get_count("ko_donations", "id", $z_where);
	$es = db_select_data("ko_donations", "WHERE 1 ".$z_where, "*", $order, $z_limit);

	$list = new kOOL_listview();

	$list->init("donations", "ko_donations", array("chk", "edit", "delete"), $_SESSION["show_start"], $_SESSION["show_limit"]);
	$list->setTitle(getLL("donations_list_title"));
	$list->setAccessRights(array("edit" => $item_edit, "delete" => $item_edit));
	$list->setActions(array("edit" => array("action" => "edit_donation"),
													"delete" => array("action" => "delete_donation", "confirm" => TRUE))
										);
	$list->setSort(TRUE, "setsort", $_SESSION["sort_donations"], $_SESSION["sort_donations_order"]);
	$list->setStats($rows);
	$list->setColumnLink("person", "index.php?action=set_person_filter&amp;id=PERSON");
	$list->setColParams(array(0 => "", 1 => "", 2 => 'style="text-align: right; padding-right: 5px"'));


	//Footer
	//Totals (get all donations without limit
	$all = db_select_data("ko_donations", "WHERE 1 ".$z_where, "*", $order);
	$total_amount = 0;
	foreach($all as $e) {
		$total_amount += $e["amount"];
		$user[$e["person"]] += 1;
	}
	//Averages
	$avg = $rows ? ko_round05($total_amount/$rows) : 0;  //ROUND
	$avg_person = sizeof($user) ? ko_round05($total_amount/sizeof($user)) : 0;  //ROUND
	
	$list_footer[] = array("label" => "", "button" => sprintf(getLL("donations_list_footer_stats_totals"), $total_amount, $rows, sizeof($user)));
	$list_footer[] = array("label" => "", "button" => sprintf(getLL("donations_list_footer_stats_averages"), $avg, $avg_person));

	$list->setFooter($list_footer);


	//Output the list
	if($output) {
		$list->render($es, $mode, getLL("donations_export_filename"));
		if($mode == "xls") return $list->xls_file;
	} else {
		print $list->render($es);
	}
}//ko_list_donations()





function ko_list_accounts($output=TRUE) {
	global $smarty;
	global $group_mod;

	if(!$group_mod) return;

	$rows = db_get_count("ko_donations_accounts", "id");
  $z_limit = "LIMIT " . ($_SESSION["show_start"]-1) . ", " . $_SESSION["show_limit"];
	$es = db_select_data("ko_donations_accounts", "", "*", "ORDER BY number ASC", $z_limit);

	$list = new kOOL_listview();

	$list->init("donations", "ko_donations_accounts", array("chk", "edit", "delete"), $_SESSION["show_start"], $_SESSION["show_limit"]);
	$list->setTitle(getLL("donations_accounts_list_title"));
	$list->setAccessRights(FALSE);
	$list->setActions(array("edit" => array("action" => "edit_account"),
													"delete" => array("action" => "delete_account", "confirm" => TRUE))
										);
	$list->setStats($rows);
	$list->setSort(FALSE);

	//Output the list
	if($output) {
		$list->render($es);
	} else {
		print $list->render($es);
	}
}//ko_list_accounts()




function ko_list_reoccuring_donations($output=TRUE) {
	global $KOTA;
	global $group_view, $group_new, $group_edit, $group_mod, $item_new, $item_edit;

	if(!$group_view) return;

	$order = "ORDER BY ".$_SESSION["sort_donations"]." ".$_SESSION["sort_donations_order"];
	$z_where = " AND reoccuring > 0 ";

	$rows = db_get_count("ko_donations", "id", $z_where);
	$es = db_select_data("ko_donations", "WHERE 1 ".$z_where, "*", $order);


	//Add new column deadline
	$KOTA["ko_donations"]["due"] = array("list" => "none");
	$KOTA["ko_donations"]["_listview"][5] = array("name" => "due");
	unset($KOTA["ko_donations"]["_listview"][10]);  //Don't show date of last donation

	//Prepare due date as new column
	foreach($es as $i => $e) {
		$due = add2date($e["date"], "tag", $e["reoccuring"], TRUE);
		if(date("Ymd") >= str_replace("-", "", $due)) {
			$pre = '<span style="color: red; font-weight: 900;">';
			$post = '</span>';
		} else {
			$pre = $post = "";
		}
		$es[$i]["due"] = $pre.sql2datum($due).$post;
		$sort[$i] = $due;
	}
	//Sort entries for due
	asort($sort);
	$new = array();
	foreach($sort as $i => $due) {
		$new[$i] = $es[$i];
	}
	$es = $new;


	$list = new kOOL_listview();

	$list->init("donations", "ko_donations", array("chk", "check", "delete"), 1, $rows);
	$list->setTitle(getLL("donations_reoccuring_list_title"));
	$list->setAccessRights(array("check" => $item_new, "delete" => $item_edit));
	$list->setAccessRights(FALSE);
	$list->setActions(array("check" => array("action" => "do_reoccuring_donation"),
													"delete" => array("action" => "delete_reoccuring_donation", "confirm" => TRUE))
										);
	$list->setSort(FALSE);
	$list->setStats($rows);

	//Footer
	$list_footer[] = array("label" => getLL("donations_list_footer_do_reoccuring"),
												 "button" => '<input type="submit" name="submit_do_reoccuring" onclick="set_action(\'do_reoccuring_donations\');this.submit;" value="'.getLL("donations_list_footer_do_reoccuring_button").'" />',
												 );

	$list->setFooter($list_footer);

	//Output the list
	if($output) {
		$list->render($es);
	} else {
		print $list->render($es);
	}
}//ko_list_reoccuring_donations()




function ko_formular_donation($mode, $id="") {
	global $KOTA;

	if($mode == "new") {
		$id = 0;
	} else if($mode == "edit") {
		if(!$id) return FALSE;
	} else {
		return FALSE;
	}

	$form_data["title"] =  $mode == "new" ? getLL("form_donation_title_new") : getLL("form_donation_title_edit");
	$form_data["submit_value"] = getLL("save");
	$form_data["action"] = $mode == "new" ? "submit_new_donation" : "submit_edit_donation";
	if($mode == 'edit') {
		$form_data["action_as_new"] = "submit_as_new_donation";
		$form_data["label_as_new"] = getLL('donations_submit_as_new');
	}
	$form_data["cancel"] = "list_donations";

	ko_multiedit_formular("ko_donations", "", $id, "", $form_data);
}//ko_formular_donation()



function ko_formular_account($mode, $id="") {
	global $KOTA;

	if($mode == "new") {
		$id = 0;
	} else if($mode == "edit") {
		if(!$id) return FALSE;
	} else {
		return FALSE;
	}

	$form_data["title"] =  $mode == "new" ? getLL("form_donation_title_new_account") : getLL("form_donation_title_edit_account");
	$form_data["submit_value"] = getLL("save");
	$form_data["action"] = $mode == "new" ? "submit_new_account" : "submit_edit_account";
	$form_data["cancel"] = "list_accounts";

	ko_multiedit_formular("ko_donations_accounts", "", $id, "", $form_data);
}//ko_formular_account()




function ko_donations_stats($output=TRUE, $mode="html") {
	global $group_mod, $smarty, $ko_path;

	if(!$group_mod) return FALSE;

	switch($_SESSION["stats_mode"]) {
		case "year":
			$years = db_select_distinct("ko_donations", "YEAR(`date`)");
			foreach($years as $key => $value) if(!$value) unset($years[$key]);

			//Get data for current year
			$data = ko_donations_get_stats_year($_SESSION["stats_year"]);

			//Row title (account)
			$a_where = "WHERE `id` IN ('".implode("','", $_SESSION["show_accounts"])."')";
			$all_accounts = db_select_data("ko_donations_accounts", $a_where, "*", "ORDER BY number ASC");
			foreach($all_accounts as $a) {
				$data["accounts"][$a["id"]]["name"] = $a["number"]." ".$a["name"];
			}
			//header
			for($m = 1; $m <= 12; $m++) {
				$header[] = strftime("%B", mktime(1,1,1, $m, 1, date("Y")));
			}

			//Export to Excel
			if($mode == "xls") {
				//Data for each account
				$xls_data = array();
				$row = 0;
				foreach($data["accounts"] as $id => $a) {
					$xls_data[$row][] = $a["name"];
					for($m=1; $m<=12; $m++) {
						$xls_data[$row][] = $a[$m]["amount"];
					}
					$xls_data[$row][] = $a["total"]["amount"];
					$row++;
				}
				//Add row with totals
				$xls_data[$row][] = getLL("total");
				for($m=1; $m<=12; $m++) {
					$xls_data[$row][] = $data["total"][$m]["amount"];
				}
				$xls_data[$row][] = $data["grand_total"]["amount"];
				//XLS Headers
				$xls_header = array_merge(array($_SESSION["stats_year"]), $header, array(getLL("total")));

				$filename = $ko_path."download/excel/".getLL("donations_export_filename").strftime("%d%m%Y_%H%M%S", time()).".xls";
				ko_export_to_excel($xls_header, $xls_data, $filename, getLL("donations_export_title"));
				return $filename;

			} else if($mode == "html") {
				//Draw BarChart
				$BCdata = array();
				$BClegend = array();
				foreach($data["total"] as $m => $values) {
					$BCdata[$m] = ko_round05($values["amount"]);
					$BClegend[$m] = strftime("%B", mktime(1,1,1, $m, 1, date("Y")));
				}
				$barChart = ko_bar_chart($BCdata, $BClegend, "", 800);
				$smarty->assign("img_year", $barChart);
				$smarty->assign("img_year_title", sprintf(getLL("donations_stats_img_year_title"), $_SESSION["stats_year"]));

				$smarty->assign("table_year_title", sprintf(getLL("donations_stats_table_year_title"), $_SESSION["stats_year"]));
				$smarty->assign("label_total", getLL("total"));
				$smarty->assign("tpl_years", $years);
				$smarty->assign("cur_year", $_SESSION["stats_year"]);
				$smarty->assign("tpl_header", $header);
				$smarty->assign("tpl_data", $data);
				$tpl_file = "ko_donations_stats.tpl";
			}
		break;  //year

		case "month":
		break;  //month
	}//switch(stats_mode

	//Output the list
	if($output) {
		$smarty->display($tpl_file);
	} else {
		print $smarty->fetch($tpl_file);
	}
}//ko_donations_stats()




function ko_donations_get_stats_year($year) {
	$a_where = "WHERE `id` IN ('".implode("','", $_SESSION["show_accounts"])."')";
	$all_accounts = db_select_data("ko_donations_accounts", $a_where, "*", "ORDER BY number ASC");
	$data = array();

	apply_donations_filter($z_where, $z_limit);
	for($m = 1; $m <= 12; $m++) {
		$all_donations = db_select_data("ko_donations", "WHERE (YEAR(`date`) = '$year' AND MONTH(`date`) = '$m') $z_where", "*");
		$total_donations = $total_amount = 0;
		$person = $donations = $amounts = array();

		foreach($all_donations as $donation) {
			$person[$donation["person"]] += 1;
			$donations[$donation["account"]] += 1;
			$amounts[$donation["account"]] += $donation["amount"];
			$total_donations += 1;
			$total_amount += $donation["amount"];
		}

		$data["total"][$m] = array("donations" => $total_donations, "amount" => $total_amount);
		foreach($all_accounts as $account) {
			$data["accounts"][$account["id"]][$m] = array("donations" => $donations[$account["id"]],
																										"amount" => $amounts[$account["id"]]);
			$data["accounts"][$account["id"]]["total"]["donations"] += $donations[$account["id"]];
			$data["accounts"][$account["id"]]["total"]["amount"] += $amounts[$account["id"]];
		}

		$data["grand_total"]["donations"] += $total_donations;
		$data["grand_total"]["amount"] += $total_amount;
	}//for(m=1..12)

	return $data;
}//ko_donations_get_stats_year()




function ko_donations_merge() {
	global $smarty;

	//Get donators
	$donators = db_select_data("ko_donations", "WHERE 1", "person, COUNT(*) as num", "GROUP by `person`");
	foreach($donators as $person) {
		ko_get_person_by_id($person["person"], $p);
		$sort_key = $p["nachname"].$p["vorname"].$person["person"];
		$d_values[$sort_key] = $person["person"];
		$d_descs[$sort_key] = $p["nachname"]." ".$p["vorname"].($p["ort"] ? (" ".getLL("from")." ".$p["ort"]) : "")." (".$person["num"].")";
	}
	ksort($d_values);
	ksort($d_descs);

	$gc = $rowcounter = 0;
	$group[$gc] = array("titel" => getLL("donations_merge_title"), "state" => "open");
	$group[$gc]["row"][$rowcounter++]["inputs"][0] = array("desc" => getLL("donations_merge_person1"),
																												 "type" => "select",
																												 "name" => "merge_person1",
																												 "values" => $d_values,
																												 "descs" => $d_descs,
																												 "params" => 'size="0"'
																												 );
	$group[$gc]["row"][$rowcounter++]["inputs"][0] = array("desc" => getLL("donations_merge_person2"),
																												 "type" => "doubleselect",
		  													 												 "js_func_add" => "double_select_add",
																												 "name" => "merge_person2",
																												 "values" => $d_values,
																												 "descs" => $d_descs,
																												 "params" => 'size="7"'
																												 );

	$smarty->assign("tpl_titel", getLL("donations_merge_title"));
	$smarty->assign("tpl_submit_value", getLL("donations_merge_submit"));
	$smarty->assign("tpl_action", "submit_merge");
	$smarty->assign("tpl_cancel", "list_donations");
	$smarty->assign("tpl_groups", $group);
	$smarty->assign("help", ko_get_help("donations", "merge"));

	$smarty->display('ko_formular.tpl');
}//ko_donations_merge()




function ko_donations_export_person() {
	global $group_view, $ko_path;

	apply_donations_filter($z_where, $z_limit);
	$address_columns = array("firm", "department", "anrede", "vorname", "nachname", "adresse", "adresse_zusatz", "plz", "ort", "land");

	//Get donators
	$donators = db_select_distinct("ko_donations", "person", "", "WHERE 1 ".$z_where);

	//Get accounts
	$a_where = "";
	foreach($_SESSION["show_accounts"] as $d) {
		if($group_view[$d]) $a_where .= " `account` = '$d' OR ";
	}
	if($a_where) $a_where = substr($a_where, 0, -3);
	else $a_where = ' 1=2 ';
	$accounts = db_select_data("ko_donations_accounts", "WHERE ".str_replace("account", "id", $a_where), "*", "ORDER BY number ASC");

	//Handle every single donator
	$rowcounter = 0;
	foreach($donators as $pid) {
		//Address data
		ko_get_person_by_id($pid, $person);
		//Sort key
		$sort_key = $person["nachname"].$person["vorname"]."_".$rowcounter;
		foreach($address_columns as $col) {
			$data[$sort_key][] = $person[$col];
		}

		$total_amount = $total_num = 0;
		//Get donations for this user for each account
		foreach($accounts as $account) {
			//Get number of donations
			$donations = db_select_data("ko_donations", "WHERE `person` = '$pid' AND `account` = '".$account["id"]."' ".$z_where, "*");
			$data[$sort_key][] = sizeof($donations);
			$total_num += sizeof($donations);
			//Sum up the amounts
			$sum = 0;
			foreach($donations as $d) $sum += $d["amount"];
			$data[$sort_key][] = sprintf("%.2f", $sum);
			$total_amount += $sum;
		}

		$data[$sort_key][] = $total_num;
		$data[$sort_key][] = $total_amount;
		$rowcounter++;
	}//foreach(donators as pid)

	ksort($data);

	//prepare header
	foreach($address_columns as $col) {
		$header[] = getLL("kota_ko_leute_".$col);
	}
	foreach($accounts as $account) {
		$header[] = $account["number"]." ".$account["name"]." (".getLL("donations_export_num").")";
		$header[] = $account["number"]." ".$account["name"]." (".getLL("donations_export_amount").")";
	}
	$header[] = getLL("total")." (".getLL("donations_export_num").")";
	$header[] = getLL("total")." (".getLL("donations_export_amount").")";

	//Export to Excel
	$filename = $ko_path."download/excel/".getLL("donations_export_filename").strftime("%d%m%Y_%H%M%S", time()).".xls";
	ko_export_to_excel($header, $data, $filename, getLL("donations_export_title"));

	return $filename;
}//ko_donations_export_person()





/**
	* Filter und Limit anwenden
	*/
function apply_donations_filter(&$z_where, &$z_limit) {
	global $group_view;

	//Accounts from itemlist
	$z_where = "";
	foreach($_SESSION["show_accounts"] as $d) {
		if($group_view[$d]) $z_where .= " `account` = '$d' OR ";
	}
	if($z_where) $z_where = " AND (".substr($z_where, 0, -3).") ";
	else $z_where = " AND 1=2 ";


	//Apply filters
	foreach($_SESSION["donations_filter"] as $key => $value) {
		if(!$value) continue;
		switch($key) {
			case "date1":
				ko_guess_date($_SESSION["donations_filter"][$key], "first");
				$z_where .= " AND `date` >= '".$_SESSION["donations_filter"][$key]."' ";
			break;

			case "date2":
				ko_guess_date($_SESSION["donations_filter"][$key], "last");
				$z_where .= " AND `date` <= '".$_SESSION["donations_filter"][$key]."' ";
			break;

			case "leute":
				if(substr($_SESSION['donations_filter'][$key], 0, 3) == '@G@') $filterset = ko_get_userpref('-1', substr($_SESSION['donations_filter'][$key], 3), 'filterset');
				else $filterset = ko_get_userpref($_SESSION["ses_userid"], $_SESSION["donations_filter"][$key], "filterset");
				$filter = unserialize($filterset[0]["value"]);
				if(TRUE === apply_leute_filter($filter, $leute_where)) {
					$leute = db_select_data("ko_leute", "WHERE 1 ".$leute_where, "id");
					if(sizeof($leute) == 0) {
						$z_where .= " AND 1=2 ";
					} else {
						$z_where .= " AND `person` IN (".implode(",", array_keys($leute)).") ";
					}
				}
			break;

			case "person":
				$z_where .= " AND `person` = '".$_SESSION["donations_filter"][$key]."' ";
			break;
		}//switch(key)
	}//foreach(SESSION[filter])

	//print "where: $z_where<br />";

	//Limit bestimmen
  $z_limit = "LIMIT " . ($_SESSION["show_start"]-1) . ", " . $_SESSION["show_limit"];
}//apply_donations_filter()




function ko_donations_settings() {
	global $smarty;
	global $group_mod;

	if(!$group_mod) return false;

	//Get filter presets for this user and the global ones to select from
	$values = array();
	$descs = array();
	$values[] = $descs[] = '';
	$filterset = array_merge((array)ko_get_userpref('-1', '', 'filterset', 'ORDER BY `key` ASC'), (array)ko_get_userpref($_SESSION['ses_userid'], '', 'filterset', 'ORDER BY `key` ASC'));
	//Get the currently stored filter
	$value = ko_get_setting('ps_filter_sel_ds1_koi[ko_donations][person][0]');
	$sel_value = '';
	foreach($filterset as $f) {
		$values[] = $f['user_id'] == '-1' ? '@G@'.$f['key'] : $f['key'];
		$descs[] = $f['user_id'] == '-1' ? getLL('itemlist_global_short').' '.$f['key'] : $f['key'];
		if($value == $f['value']) $sel_value = $f['user_id'] == '-1' ? '@G@'.$f['key'] : $f['key'];
	}

	//build form
	$gc = 0;
	$rowcounter = 0;
	$frmgroup[$gc]['row'][$rowcounter++]['inputs'][1] = array('desc' => getLL('donations_settings_ps_filter'),
		'type' => 'select',
		'name' => 'sel_ps_filter',
		'values' => $values,
		'descs' => $descs,
		'value' => $sel_value,
	);

	//display the form
	$smarty->assign('tpl_titel', getLL('donations_settings_form_title'));
	$smarty->assign('tpl_submit_value', getLL('save'));
	$smarty->assign('tpl_action', 'submit_settings');
	$cancel = ko_get_userpref($_SESSION['ses_userid'], 'default_view_donations');
	if(!$cancel) $cancel = 'list_donations';
  $smarty->assign('tpl_cancel', $cancel);
	$smarty->assign('tpl_groups', $frmgroup);

	$smarty->display('ko_formular.tpl');

}//ko_donation_settings()

?>
