<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2003-2010 Renzo Lauper (renzo@churchtool.org)
*  All rights reserved
*
*  This script is part of the kOOL project. The kOOL project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*  A copy is found in the textfile GPL.txt and important notices to the license
*  from the author is found in LICENSE.txt distributed with these scripts.
*
*  kOOL is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once($BASE_PATH.'inc/class.kOOL_listview.php');




/**
 * List currently available trackings
 */
function ko_list_trackings($output=TRUE) {
	global $smarty;
	global $group_view, $group_new, $group_edit, $group_mod;

	if(!$group_view) return;
	apply_tracking_filter($z_where, $z_limit, 'ko_tracking');

	//Apply selection in itemlist
	if(sizeof($_SESSION['show_tracking_groups']) > 0) {
		$z_where = "AND `group_id` IN ('".implode("','", $_SESSION['show_tracking_groups'])."')";
	} else {
		$z_where = 'AND 1=2';
	}

	$order = 'ORDER BY '.$_SESSION['sort_trackings'].' '.$_SESSION['sort_trackings_order'];

	$rows = db_get_count('ko_tracking', 'id', $z_where);
	$es = db_select_data('ko_tracking', 'WHERE 1=1 '.$z_where, '*', $order, $z_limit);
	//Check for view rights, otherwise don't show tracking
	foreach($es as $tid => $t) {
		if(!$group_view[$tid]) {
			unset($es[$tid]);
			$rows--;
		}
	}

	$list = new kOOL_listview();

	$list->init('tracking', 'ko_tracking', array('chk', 'edit', 'delete'), $_SESSION['show_start'], $_SESSION['show_limit']);
	$list->setTitle(getLL('tracking_list_title'));
	$list->setAccessRights(array('edit' => $group_edit, 'delete' => $group_mod));
	$list->setActions(array('edit' => array('action' => 'edit_tracking'),
													'delete' => array('action' => 'delete_tracking', 'confirm' => TRUE))
										);
	$list->setSort(TRUE, 'setsort', $_SESSION['sort_tracking'], $_SESSION['sort_tracking_order']);
	$list->setStats($rows);
	$list->setColumnLink('name', 'index.php?action=enter_tracking&amp;id=ID');


	//Output the list
	if($output) {
		$list->render($es);
	} else {
		print $list->render($es);
	}
}//ko_list_trackings()




/**
 * Updates all tracking date settings to only store the ones fitting the selected date_mode
 * Called after every saving of a tracking
 */
function ko_tracking_update_dates() {
	db_update_data('ko_tracking', "WHERE `date_mode` = 'dates'", array('date_eventgroup' => '', 'date_weekdays' => ''));
	db_update_data('ko_tracking', "WHERE `date_mode` = 'eventgroup'", array('dates' => '', 'date_weekdays' => ''));
	db_update_data('ko_tracking', "WHERE `date_mode` = 'weekdays'", array('date_eventgroup' => '', 'dates' => ''));
}//ko_tracking_update_dates()





/**
 * Show form to enter and edit trackings. Uses fields as defined in KOTA
 */
function ko_formular_tracking($mode, $id='') {
	global $KOTA, $group_mod;

	if($mode == 'new') {
		$id = 0;
	} else if($mode == 'edit') {
		if(!$id) return FALSE;
	} else {
		return FALSE;
	}

	$form_data['title'] =  $mode == 'new' ? getLL('tracking_form_title_new') : getLL('tracking_form_title_edit');
	$form_data['submit_value'] = getLL('save');
	$form_data['action'] = $mode == 'new' ? 'submit_new_tracking' : 'submit_edit_tracking';
	if($mode == 'edit' && $group_mod[$id]) {
		$form_data['action_as_new'] = 'submit_as_new_tracking';
		$form_data['label_as_new'] = getLL('tracking_form_submit_as_new');
	}
	$form_data['cancel'] = 'list_trackings';

	ko_multiedit_formular('ko_tracking', '', $id, '', $form_data);
}//ko_formular_tracking()






/**
 * Show form to enter tracking
 */
function ko_tracking_enter_form($output=TRUE) {
	global $smarty, $group_view, $group_new, $DATETIME;

	$tracking = db_select_data('ko_tracking', 'WHERE `id` = \''.$_SESSION['tracking_id'].'\'', '*', '', '', TRUE);

	$people = ko_tracking_get_people($tracking['filter']);
	$dates = ko_tracking_get_dates($tracking, '', '', $prev, $next);
	$raw_dates = array();
	foreach($dates as $date) {
		$raw_dates[] = $date['date'];
	}

	//Get all currently stored entries (only for people currently active for this tracking)
	$entries = array();
	$where = "WHERE `tid` = '".$tracking['id']."' AND `date` IN ('".implode("','", $raw_dates)."') AND `lid` IN ('".implode("','", array_keys($people))."')";
	$rows = db_select_data('ko_tracking_entries', $where, '*', 'ORDER BY lid,date ASC');
	foreach($rows as $row) {
		if($tracking['mode'] == 'type') {
			$entries[$row['lid']][$row['date']][$row['type']] = $row['value'];
		} else {
			$entries[$row['lid']][$row['date']] = $row['value'];
		}
		$sums[$row['date']][] = $row['value'];
	}

	//Build sums for each person over the whole active month
	$sum_where = "WHERE `tid` = '".$tracking['id']."' AND YEAR(`date`) = '".substr($_SESSION['date_start'], 0, 4)."' AND MONTH(`date`) = '".substr($_SESSION['date_start'], 5, 2)."' AND `lid` IN ('".implode("','", array_keys($people))."')";
	$sum_rows = db_select_data('ko_tracking_entries', $sum_where);
	$psums = array();
	foreach($sum_rows as $row) {
		switch($tracking['mode']) {
			case 'simple':
				if($row['value'] == 1) $psums[$row['lid']] += 1;
			break;
			case 'value':
				if(is_numeric($row['value'])) {
					$psums[$row['lid']]['numeric'] += (float)$row['value'];
				} else {
					$psums[$row['lid']][$row['value']] += 1;
				}
			break;
			case 'type':
				$psums[$row['lid']] += (float)$row['value'];
			break;
		}
	}
	foreach($people as $id => $p) {
		if($tracking['mode'] == 'value') {
			$psum = '';
			if($psums[$id]['numeric'] > 0) $psum = $psums[$id]['numeric'];
			if(sizeof($psums[$id]) > 0) {
				foreach($psums[$id] as $k => $v) {
					if($k == 'numeric') continue;
					$psum .= ', '.$v.'x'.$k;
				}
			}
			if(substr($psum, 0, 2) == ', ') $psum = substr($psum, 2);
			$people[$id]['_sum'] = $psum;
		} else {
			if($psums[$id] > 0) $people[$id]['_sum'] = $psums[$id];
		}
	}

	//Build sums for each date
	if($tracking['mode'] == 'simple') {
		foreach($sums as $date => $values) {
			$s = 0;
			foreach($values as $value) {
				if($value) $s++;
			}
			$sum[$date] = $s;
		}
	} elseif($tracking['mode'] == 'value' || $tracking['mode'] == 'type') {
		foreach($sums as $date => $values) {
			$s1 = $s2 = $s3 = 0;
			foreach($values as $value) {
				if(!$value) continue;
				if(is_numeric($value)) $s1 += $value;
				else if(!is_numeric($value)) $s2++;
				$s3++;
			}
			$sum[$date] = $s2 > 0 ? $s3 : $s1;
		}
	}

	//Add type select
	if($tracking['mode'] == 'type') {
		$types = array();
		foreach(explode("\n", $tracking['types']) as $t) {
			if(!$t) continue;
			$types[] = trim($t);
		}
		$smarty->assign('types', $types);
	}

	//Create month/year select for navigation
	$today = date('Y-m-d');
	$dateselect['months'] = array();
	for($i=-24; $i<=12; $i++) {
		$date = add2date($today, 'month', $i, TRUE);
		$dateselect['months'][] = array('value' => substr($date, 0, 7).'-01', 'desc' => strftime($DATETIME['nY'], strtotime($date)));
	}
	$dateselect['selected'] = substr($_SESSION['date_start'], 0, 7).'-01';
	$dateselect['today'] = substr($today, 0, 7).'-01';

	$smarty->assign('sums', $sum);
	$smarty->assign('label_total', getLL('tracking_list_total'));
	$smarty->assign('total', sizeof($people));

	$smarty->assign('label_name', getLL('tracking_list_name'));
	$smarty->assign('people', $people);
	$smarty->assign('tracking', $tracking);
	$smarty->assign('dates', $dates);
	$smarty->assign('entries', $entries);
	//Set links for date navigation
	$smarty->assign('next_date', $next);
	$smarty->assign('prev_date', $prev);
	$smarty->assign('today_date', date('Y-m-d'));
	$smarty->assign('dateselect', $dateselect);

	//Set form to readonly if appropriate rights are not given
	$smarty->assign('readonly', !$group_new[$_SESSION['tracking_id']]);

	//Help link
	$smarty->assign('help', ko_get_help('tracking', 'enter_tracking'));

	$smarty->assign('module', 'tracking');
	$smarty->assign('sesid', session_id());
	$smarty->assign('label_del_confirm', getLL('list_label_confirm_delete'));
	$smarty->assign('label_sum', strftime($DATETIME['nY'], strtotime($_SESSION['date_start'])));

	//Output the list
	if($output) {
		$smarty->display('ko_formular_tracking.tpl');
	} else {
		print $smarty->fetch('ko_formular_tracking.tpl');
	}
}//ko_tracking_enter_form()





/**
 * Does the date calculations for the different date methods
 *
 * @param array tracking: Tracking to get dates for
 * @param date start: If set this date will be used as first date, otherwise _SESSION[date_start] will be used
 * @param int limit: Set a maximum number of dates to return. If not set, _SESSION[date_limit] will be used
 * @param date &prev: Value for backwards date navigation
 * @param date &next: Value for forward date navigation
 * @param boolean apply_filter: Defines if filters from session should be applied or not, default to TRUE
 * @return array dates: Returns an array of dates fulfilling the given parameters
 */
function ko_tracking_get_dates($tracking, $_start='', $_limit='', &$prev, &$next, $apply_filter=TRUE) {
	global $DATETIME;

	$start = $_start ? $_start : $_SESSION['date_start'];
	$limit = $_limit ? $_limit : $_SESSION['date_limit'];

	$dates = array();
	switch($tracking['date_mode']) {
		case 'eventgroup':
			//TODO: More than one event on a date?
			apply_tracking_filter($z_where, $z_limit, 'ko_event');
			$where = "WHERE `eventgruppen_id` = '".$tracking['date_eventgroup']."' ";
			$where .= $apply_filter ? $z_where : '';
			$eg_dates = db_select_data('ko_event', $where, '*', 'ORDER BY `startdatum` ASC');
			foreach($eg_dates as $d) {
				$_dates[] = $d['startdatum'];
			}
		break;

		case 'weekdays':
			//Get limit dates forward from start
			$date = $start != -1 ? $start : $_SESSION['date_start'];
			$num_dates = 0;
			//set max number of dates beginning with start
			if($apply_filter && $_SESSION['tracking_filter']['date1'] && $_SESSION['tracking_filter']['date2']) {
				//Get all dates in between date1 and date2 of filter  
				$days = (strtotime($_SESSION['tracking_filter']['date2']) - strtotime($_SESSION['tracking_filter']['date1']))/60/60/24;
				$max = ceil($days/7*sizeof(explode(',', $tracking['date_weekdays'])))+1;
			} else {
				//2*limit to show the current and calculate the link to the next page
				$max = 2*$limit;
			}
			while($num_dates < $max) {
				if(in_array(strftime('%w', strtotime($date)), explode(',', $tracking['date_weekdays']))) {
					$_dates[] = $date;
					$num_dates++;
				}
				$date = add2date($date, 'tag', 1, TRUE);
			}

			//Get limit dates backwards from start
			$date = $start != -1 ? $start : $_SESSION['date_start'];
			$num_dates = 0;
			while($num_dates < ceil($max/2)) {
				$date = add2date($date, 'tag', -1, TRUE);
				if(in_array(strftime('%w', strtotime($date)), explode(',', $tracking['date_weekdays']))) {
					$_dates[] = $date;
					$num_dates++;
				}
			}
		break;

		case 'dates':
			$_dates = explode(',', $tracking['dates']);
		break;
	}//switch(mode)


	//Apply filter
	if($apply_filter && ($_SESSION['tracking_filter']['date1'] || $_SESSION['tracking_filter']['date2'])) {
		$fd1 = $_SESSION['tracking_filter']['date1'] ? strftime('%Y-%m-%d', strtotime($_SESSION['tracking_filter']['date1'])) : '1900-01-01';
		$fd2 = $_SESSION['tracking_filter']['date2'] ? strftime('%Y-%m-%d', strtotime($_SESSION['tracking_filter']['date2'])) : '2100-01-01';
		$__dates = array();
		foreach($_dates as $date) {
			if((int)str_replace('-', '', $date) >= (int)str_replace('-', '', $fd1)
			  && (int)str_replace('-', '', $date) <= (int)str_replace('-', '', $fd2)) {
				$__dates[] = $date;
			}
		}
		$_dates = $__dates;
	}


	if(sizeof($_dates) == 0) return array();

	$dates = array();
	$next = FALSE;
	asort($_dates);
	//Reset start to first date if filter is not be applied (e.g. for export mode all)
	if(!$apply_filter || $start == -1) $start = $_dates[0];

	$num_dates = 0;
	foreach($_dates as $date) {
		if(!$date) continue;
		if((int)str_replace('-', '', $date) >= (int)str_replace('-', '', $start)) {
			if($num_dates < $limit) {
				$dates[] = array('date' => $date, 'title' => strftime($DATETIME['dmy'], strtotime($date)));
				$num_dates++;
			} else if(!$next) {
				$next = $date;
			}
		}
	}
	//If no events found after start
	if(sizeof($dates) == 0) {
		arsort($_dates);
		$num_dates = 0;
		foreach($_dates as $date) {
			if(!$date) continue;
			if($num_dates < $limit && (int)str_replace('-', '', $date) < (int)str_replace('-', '', $start)) {
				array_unshift($dates, array('date' => $date, 'title' => strftime($DATETIME['dmy'], strtotime($date))));
				$num_dates++;
			}
		}
		//Reset start for the following search for the link to the previous page
		$start = $dates[0]['date'];
	}

	//Find link to previous page
	arsort($_dates);
	$prev = $start;
	$num_dates = 0;
	foreach($_dates as $date) {
		if(!$date) continue;
		if($num_dates < $limit && (int)str_replace('-', '', $date) < (int)str_replace('-', '', $prev)) {
			$prev = $date;
			$num_dates++;
		}
	}
	if($prev == $start) $prev = FALSE;

	return $dates;
}//ko_tracking_get_dates()





/**
 * Get members of a tracking
 *
 * @param mixed filter: A trackings filter (from DB). Can be group id, small group id or serialized array of filter preset
 * @return array people: Returns an array of addresses
 */
function ko_tracking_get_people($filter) {
	$where = " AND `deleted` = '0'";
	$mode = '';

	//Group ID
	if(substr($filter, 0, 1) == 'g' && strlen($filter) == 7) {
		$mode = 'group';
		$gid = $filter;
		$where .= " AND `groups` REGEXP '$gid'";
		//TODO: Add role?
		//$where .= " AND `groups` REGEXP '$gid[g:0-9]*$rid'";
	}
	//Small group
	else if(strlen($filter) == 4) {
		$mode = 'kg';
		$where .= " AND `kg` REGEXP '$filter'";
	}
	//Serialized filter preset array
	else {
		$mode = 'filter';
		$fa = unserialize($filter);
		if(is_array($fa)) {
			apply_leute_filter($fa, $where, FALSE);
		} else {
			$where = ' AND 1=2';
		}
	}

	ko_get_leute($people, $where, '', '', 'ORDER BY `nachname` ASC, `vorname` ASC');

	//Order group members by their role as set for the group
	if($mode == 'group') {
		$new = array();
		$group = db_select_data('ko_groups', 'WHERE `id` = \''.substr($gid, 1).'\'', '*', '', '', TRUE);
		//Use the order of the roles as they're set for the group
		foreach(explode(',', $group['roles']) as $rid) {
			foreach($people as $pid => $person) {
				$role = substr($person['groups'], strpos($person['groups'], $gid)+9, 6);
				if($rid == $role) {
					$new[$pid] = $person;
					unset($people[$pid]);
				}
			}
		}
		//Add all with no role
		foreach($people as $pid => $person) {
			$new[$pid] = $person;
		}
		$people = $new;
	}

	return $people;
}//ko_tracking_get_people()






/**
 * Creates a tracking export as XLS or PDF
 *
 * @param string mode: can be xls or pdf
 * @param string filename: Filename to be used for the output file
 * @param int id: ID of tracking to be used for export
 * @param string address_columns: Default to 'name' which just exports first and last name. May also be a column preset
 * @param string dates: May be current, filter or all and defines which dates should be used for the export
 * @param boolean combine_families: If set to TRUE family members' tracking data will be combined into one entry
 * @return int error: If there is an error during the LaTeX conversion to PDF an error code will be returned
 */
function ko_tracking_export($mode, $filename, $id, $address_columns='name', $dates='current', $combine_families=FALSE) {
	global $ko_path, $BASE_PATH, $BASE_URL, $DATETIME;

	//Get tracking, dates and people
	$tracking = db_select_data('ko_tracking', 'WHERE `id` = \''.$id.'\'', '*', '', '', TRUE);
	$people = ko_tracking_get_people($tracking['filter']);
	switch($dates) {
		case 'current':
			$dates = ko_tracking_get_dates($tracking, '', '', $prev, $next);
		break;
		case 'filter':
			if($_SESSION['tracking_filter']['date1'] && $_SESSION['tracking_filter']['date2']) {
				$dates = ko_tracking_get_dates($tracking, $_SESSION['tracking_filter']['date1'], 10000, $prev, $next);
			} else if($_SESSION['tracking_filter']['date1']) {
				$dates = ko_tracking_get_dates($tracking, $_SESSION['tracking_filter']['date1'], 250, $prev, $next);
			} else if($_SESSION['tracking_filter']['date2']) {
				$dates = ko_tracking_get_dates($tracking, -1, 250, $prev, $next);
			} else {
				$dates = ko_tracking_get_dates($tracking, '', '', $prev, $next);
			}
		break;
		case 'all':
			$dates = ko_tracking_get_dates($tracking, '', 250, $prev, $next, FALSE);
		break;
		//Get current dates by default (continuos without a filter doesn't give any options to choose from)
		default:
			$dates = ko_tracking_get_dates($tracking, '', '', $prev, $next);
	}

	//Get address columns from selected preset
	if($address_columns == 'name') {
		$columns = array('vorname', 'nachname');
	} else if(substr($address_columns, 0, 4) == 'set_') {
		$preset = substr($address_columns, 4);
		$userid = substr($preset, 0, 3) == '@G@' ? '-1' : $_SESSION['ses_userid'];
		$row = ko_get_userpref($userid, str_replace('@G@', '', $preset), 'leute_itemset');
		$columns = explode(',', $row[0]['value']);
		if(sizeof($columns) == 0) $columns = array('vorname', 'nachname');
	} else return FALSE;

	//Get tracking entries
	$entries = array();
	$rows = db_select_data('ko_tracking_entries', "WHERE `tid` = '".$tracking['id']."' AND `lid` IN ('".implode("','", array_keys($people))."')", '*', 'ORDER BY lid,date ASC');
	foreach($rows as $row) {
		if($tracking['mode'] == 'type') {
			$entries[$row['lid']][$row['date']][$row['type']] = $row['value'];
		} else {
			$entries[$row['lid']][$row['date']] = $row['value'];
		}
	}

	//Header
	$colnames = ko_get_leute_col_name();

	//Combine output by family
	if($combine_families) {
		$done_famids = array();
		$sort = array();
		$data = array();
		$row = 0;

		//Group all entries by family
		$famdata = array();
		foreach($people as $pid => $p) {
			if(!$p['famid']) continue;
			foreach($dates as $date) {
				if($entries[$p['id']][$date['date']]) $famdata[$p['famid']][$date['date']][] = array('role' => $p['famfunction'], 'value' => $entries[$p['id']][$date['date']]);
			}
		}

		foreach($people as $pid => $p) {
			if(!$p['famid']) {
				foreach($columns as $col) {
					$data[$row][] = $p[$col];
				}
				$sums = array();
				foreach($dates as $date) {
					if($tracking['mode'] == 'simple') {
						$value = $entries[$p['id']][$date['date']] > 0 ? 'X' : '';
						$sums['v'] += $entries[$p['id']][$date['date']];
					} else if($tracking['mode'] == 'value') {
						$value = $entries[$p['id']][$date['date']];
						if($value) {
							if(is_numeric($value)) $sums['v'] += $value;
							else $sums['string'][] = $value;
						}
					} else if($tracking['mode'] == 'type') {
						$value = '';
						foreach($entries[$p['id']][$date['date']] as $t => $v) {
							if(!$v || !$t) continue;
							$value .= $v.' '.$t."\n";
							(float)$sums['v'] += (float)$v;
						}
					}
					$data[$row][] = trim($value);
				}
				//Add sum column
				$value = array();
				if($sums['v']) $value[] = $sums['v'];
				if(sizeof($sums['string']) > 0) $value[] = implode(', ', $sums['string']);
				$data[$row][] = implode(', ', $value);

				$row++;
				continue;
			}

			if(in_array($p['famid'], $done_famids)) continue;

			$fam = ko_get_familie($p['famid']);
			foreach($columns as $col) {
				switch($col) {
					case 'anrede': $p[$col] = $fam['famanrede'] ? $fam['famanrede'] : getLL('ko_leute_anrede_family'); break;
					case 'vorname':
						if(!in_array('anrede', $columns)) {
							$p[$col] = $fam['famanrede'] ? $fam['famanrede'] : getLL('ko_leute_anrede_family');
						} else {
							if($fam['famfirstname']) {
								$p[$col] = $fam['famfirstname'];
							} else {
								//use first names of parents for firstname-col
								$parents = db_select_data('ko_leute', "WHERE `famid` = '".$p['famid']."' AND `famfunction` IN ('husband', 'wife')", 'famfunction,vorname', 'ORDER BY famfunction ASC');
								$parent_values = array();
								foreach($parents as $parent) $parent_values[] = $parent['vorname'];
								$p[$col] = implode((' '.getLL('family_link').' '), $parent_values);
							}
						}
					break;
					case 'nachname': $p[$col] = $fam['famlastname'] ? $fam['famlastname'] : $p['nachname']; break;
				}//switch(col)
			}

			foreach($columns as $col) {
				$data[$row][] = $p[$col];
			}
			$sums = array();
			foreach($dates as $date) {
				$values = array();

				//Set values and sums for each date for the given family
				$value = array();
				switch($tracking['mode']) {
					case 'simple':
						foreach($famdata[$p['famid']][$date['date']] as $d) {
							$key = in_array($d['role'], array('husband', 'wife')) ? 'p' : 'c';
							$values[$key] += $d['value'];
						}
						if($values['p'] > 0) $value[] = $values['p'].getLL('tracking_export_family_parents_short');
						if($values['c'] > 0) $value[] = $values['c'].getLL('tracking_export_family_children_short');
						$sums['p'] += $values['p'];
						$sums['c'] += $values['c'];
					break;
					case 'value':
						foreach($famdata[$p['famid']][$date['date']] as $d) {
							$key = in_array($d['role'], array('husband', 'wife')) ? 'p' : 'c';
							if(is_numeric($d['value'])) $values[$key]['numeric'] += $d['value'];
							else $values[$key][$d['value']] += 1;
						}
						//Handle numeric values first
						if($values['p']['numeric'] > 0) $value[] = $values['p']['numeric'].getLL('tracking_export_family_parents_short');
						if($values['c']['numeric'] > 0) $value[] = $values['c']['numeric'].getLL('tracking_export_family_children_short');
						$sums['numeric']['p'] += $values['p']['numeric'];
						$sums['numeric']['c'] += $values['c']['numeric'];
						//Handle non-numeric values
						if(sizeof($values['p']) > 0) {
							foreach($values['p'] as $k => $v) {
								if($k == 'numeric') continue;
								$value[] = $v.'x'.$k.getLL('tracking_export_family_parents_short');
								$sums[$k]['p'] += $v;
							}
						}
						if(sizeof($values['c']) > 0) {
							foreach($values['c'] as $k => $v) {
								if($k == 'numeric') continue;
								$value[] = $v.'x'.$k.getLL('tracking_export_family_children_short');
								$sums[$k]['c'] += $v;
							}
						}
					break;
					case 'type':
						foreach($famdata[$p['famid']][$date['date']] as $dd) {
							foreach($dd['value'] as $t => $d) {
								$values[$t] += $d;
							}
						}
						foreach($values as $t => $v) {
							$value[] = $v.' '.$t;
							$sums['p'] += $v;
						}
					break;
				}
				$data[$row][] = implode(', ', $value);
			}
			//Add sum column
			$sum = array();
			if($tracking['mode'] == 'value') {
				if($sums['numeric']['p']) $sum[] = $sums['numeric']['p'].getLL('tracking_export_family_parents_short');
				if($sums['numeric']['c']) $sum[] = $sums['numeric']['c'].getLL('tracking_export_family_children_short');
				if(sizeof($sums) > 0) {
					foreach($sums as $k => $d) {
						if($k == 'numeric') continue;
						foreach($d as $pc => $v) {
							$llpc = $pc == 'p' ? getLL('tracking_export_family_parents_short') : getLL('tracking_export_family_children_short');
							$sum[] = $v.'x'.$k.$llpc;
						}
					}
				}
			} else if($tracking['mode'] == 'type') {  //Don't include p/c for type sums
				if($sums['p']) $sum[] = $sums['p'];
			} else {
				if($sums['p']) $sum[] = $sums['p'].getLL('tracking_export_family_parents_short');
				if($sums['c']) $sum[] = $sums['c'].getLL('tracking_export_family_children_short');
			}
			$data[$row][] = implode(', ', $sum);

			$done_famids[] = $p['famid'];
			$row++;
		}//foreach(people as p)
	}//if(combine_families)
	else {
		//Fill tracking data
		$data = array();
		$row = 0;
		foreach($people as $p) {
			foreach($columns as $col) {
				$data[$row][] = $p[$col];
			}
			$sums = array();
			foreach($dates as $date) {
				switch($tracking['mode']) {
					case 'simple':
						$value = $entries[$p['id']][$date['date']] > 0 ? 'X' : '';
						if($value) $sums['numeric'] += 1;
					break;
					case 'value':
						$value = $entries[$p['id']][$date['date']];
						if($value) {
							if(is_numeric($value)) $sums['numeric'] += $value;
							else $sums[$value] += 1;
						}
					break;
					case 'type':
						$value = '';
						foreach($entries[$p['id']][$date['date']] as $t => $v) {
							if(!$v || !$t) continue;
							$value .= $v.' '.$t."\n";
							(float)$sums['numeric'] += (float)$v;
						}
					break;
				}
				$data[$row][] = trim($value);
			}
			//Add sum column
			if($tracking['mode'] == 'value') {
				$sum = '';
				if($sums['numeric'] > 0) $sum = $sums['numeric'];
				if(sizeof($sums) > 0) {
					foreach($sums as $k => $v) {
						if($k == 'numeric') continue;
						$sum .= ', '.$v.'x'.$k;
					}
				}
				if(substr($sum, 0, 2) == ', ') $sum = substr($sum, 2);
				$data[$row][] = $sum;
			} else {
				if($sums['numeric'] > 0) $data[$row][] = $sums['numeric'];
			}


			$row++;
		}
	}



	//Create XLS file
	if($mode == 'xls') {
		//XLS header
		$col_headers = array();
		foreach($columns as $col) {
			$col_headers[] = $colnames[$col] ? $colnames[$col] : $col;
		}
		foreach($dates as $date) {
			$col_headers[] = $date['title'];
		}
		$col_headers[] = getLL('tracking_list_total');

		//Create subtitles
		$subtitles = array();
		if(substr($tracking['filter'], 0, 1) == 'g' && strlen($tracking['filter']) == 7) {
			$group = db_select_data('ko_groups', "WHERE `id` = '".substr($tracking['filter'], 1)."'", '*', '', '', TRUE);
			$subtitles[getLL('groups_listheader_name')] = $group['name'];
		} else if(strlen($tracking['filter']) == 4) {
			ko_get_kleingruppen($_kg, '', $tracking['filter']);
			$kg = $_kg[$tracking['filter']];
			if($kg['name']) $subtitles[getLL('kg_listheader_name')] = $kg['name'];
			if($kg['kg_leiter']) {
				$leader = array();
				foreach(explode(',', $kg['kg_leiter']) as $pid) {
					ko_get_person_by_id($pid, $p);
					$leader[] = $p['vorname'].' '.$p['nachname'];
				}
				$subtitles[getLL('kg_listheader_kg_leiter')] = implode(', ', $leader);
			}
			if($kg['wochentag']) $subtitles[getLL('kg_listheader_wochentag')] = getLL('ko_kleingruppen_wochentag_'.$kg['wochentag']);
			if($kg['ort']) $subtitles[getLL('kg_listheader_ort')] = $kg['ort'];
			if($kg['zeit']) $subtitles[getLL('kg_listheader_zeit')] = getLL('time_at').' '.$kg['zeit'];
		} else {
		}

		$header = array('header' => $col_headers, 'title' => $tracking['name'], 'subtitle' => $subtitles);
		ko_export_to_excel($header, $data, $filename, $tracking['name']);
	}

	//Create PDF file
	else if($mode == 'pdf') {
		//Create mapping array for labels and titles
		$map = array();
		$map['###LABEL_NAME###'] = getLL('tracking_export_label_name');
		$map['###LABEL_TIMESPAN###'] = getLL('tracking_export_label_timespan');
		$map['###LABEL_PAGE###'] = getLL('tracking_export_label_page');
		$person = ko_get_logged_in_person();
		$creator = $person['vorname'] ? $person['vorname'].' '.$person['nachname'] : $_SESSION['ses_username'];
		$map['###CREATED###'] = sprintf(getLL('tracking_export_label_created'), strftime($DATETIME['dmY'].' %H:%M', time()), $creator);
		$map['###NAME###'] = $tracking['name'];
		$map['###TIMESPAN###'] = strftime($DATETIME['dmY'], strtotime($dates[0]['date'])).' - '.strftime($DATETIME['dmY'], strtotime($dates[sizeof($dates)-1]['date']));
		$map['###KOOL_URL###'] = $BASE_URL;

		//Map latex chars
		foreach($map as $key => $value) {
			$map[$key] = ko_latex_escape_chars($value);
		}
		//Add logo
		$found = FALSE;
		$open = @opendir($ko_path.'my_images/');
		while($file = @readdir($open)) {
			if(preg_match('/^pdf_logo\.(png|jpg|gif)$/', $file)) {
				$found = TRUE;
				$logo_file = $file;
			}
		}
		if($found) {
			$map['###LOGO###'] = '\rhead{\includegraphics[height=20mm]{../../my_images/'.$logo_file.'}}';
		} else {
			$map['###LOGO###'] = '';
		}

		$data_width = $tracking['mode'] == 'type' ? 3 : 1;
		$default_cols = $tracking['mode'] == 'type' ? 6 : 14;

		//Calculate maximum number of data columns to fit on the page
		$num_cols = sizeof($columns);
		$num_dates = sizeof($dates);
		$max = floor($default_cols-($num_cols-2)*2.5);
		if($num_dates > $max) {
			$tables = ceil((float)$num_dates/(float)$max);
		} else {
			$tables = 1;
		}

		$table = '';
		$counter = 0;
		for($t=1; $t<=$tables; $t++) {
			$ldates = array();
			$ldata = array();
			$start = $counter; 
			$stop = min($counter+$max, $num_dates);
			for($i=$start; $i<$stop; $i++) {
				$ldates[] = ko_latex_escape_chars(strftime($DATETIME['dm'], strtotime($dates[$i]['date'])));;
			}
			foreach($data as $key => $row) {
				for($j=0; $j<$num_cols; $j++) {
					$ldata[$key][] = $row[$j];
				}
				for($i=($num_cols+$start); $i<($num_cols+$stop); $i++) {
					$ldata[$key][] = str_replace("\n", '\par', ko_latex_escape_chars($row[$i]));
				}
				//Add sum for last table
				if($t == $tables) {
					$ldata[$key][] = $row[$num_cols+$stop];
				}
			}
			//Create table header
			$table_cols = array();
			$table_header = array();
			foreach($columns as $col) {
				$table_cols[] = 'p{3cm}';
				$table_header[] = '\textbf{'.ko_latex_escape_chars(($colnames[$col] ? $colnames[$col] : $col)).'}';
			}
			foreach($ldates as $date) {
				$table_cols[] = 'p{'.$data_width.'.cm}';
				$table_header[] = '\textbf{'.$date.'}';
			}
			//Add sum column header for last table
			if($t == $tables) {
				$table_cols[] = 'p{1cm}';
				$table_header[] = '$\sum$';
			}
			$table .= '\begin{longtable}[l]{'.implode('|', $table_cols).'}'."\n";
			$table .= '\hline'."\n";
			$table .= implode('&', $table_header).'\endhead'."\n";
			$table .= '\hline'."\n";

			//Fill in table data
			$table_content = '';
			foreach($ldata as $row) {
				$table_content .= implode('&', $row).'\\\\'."\n";
				$table_content .= '\hline'."\n";
			}
			$map['###TABLE_CONTENT###'] = $table_content;
			$table .= $table_content."\n";
			$table .= '\end{longtable}'."\n";

			$counter += $max;
		}//for(t=1..tables)
		$map['###TABLE###'] = $table;

		//Create LaTeX file
		$latex = file_get_contents($ko_path.'latex/layouts/tracking.tex');
		$latex = strtr($latex, $map);

		//Convert LaTeX to PDF
		$file = md5(uniqid(mt_rand(), true));
		if(!$fh = fopen($BASE_PATH.'latex/compile/'.$file.'.tex', 'w')) {
			return 1;
		}
		if(!fwrite($fh, $latex)) {
			return 1;
		}
		//Compile twice to get table alignments correct
		system('cd '.$BASE_PATH.'latex/compile/ && pdflatex -interaction nonstopmode '.$file.'.tex 2&>/dev/null', $ret);
		system('cd '.$BASE_PATH.'latex/compile/ && pdflatex -interaction nonstopmode '.$file.'.tex 2&>/dev/null', $ret);
		foreach(array('.aux', '.log', '.tex') as $type) {
			$f = $BASE_PATH.'latex/compile/'.$file.$type;
			if(is_file($f)) unlink($f);
		}
		if(is_file($ko_path.'latex/compile/'.$file.'.pdf')) {
			copy($ko_path.'latex/compile/'.$file.'.pdf', $filename);
			unlink($ko_path.'latex/compile/'.$file.'.pdf');
		} else {
			return 2;
		}
	}
}//ko_tracking_export_excel()







/**
	* Apply filter and limit for tracking dates
	*/
function apply_tracking_filter(&$z_where, &$z_limit, $table) {
	global $group_view;

	$field = $table == 'ko_event' ? 'startdatum' : 'date';

	//Apply filters
	foreach($_SESSION['tracking_filter'] as $key => $value) {
		if(!$value) continue;
		switch($key) {
			case 'date1':
				ko_guess_date($_SESSION['tracking_filter'][$key], 'first');
				$z_where .= " AND `$field` >= '".$_SESSION['tracking_filter'][$key]."' ";
			break;

			case 'date2':
				ko_guess_date($_SESSION['tracking_filter'][$key], 'last');
				$z_where .= " AND `$field` <= '".$_SESSION['tracking_filter'][$key]."' ";
			break;
		}//switch(key)
	}//foreach(SESSION[filter])

	//Limit bestimmen
  $z_limit = 'LIMIT '.($_SESSION['show_start']-1).', '.$_SESSION['show_limit'];
}//apply_tracking_filter()

?>
